'use strict';

/**
 * 这是一个内部模块. 查看 {@link FinoChatClient} 这个对外公共类.
 * @module client
 */

var EventEmitter = require('events').EventEmitter;
var utils = require('./utils');
var FinoChatBaseApis = require('./base-apis');
var log = require('./log');
var jsBridgeEvent = require('./events').jsBridgeEvents;

/**
 * 创建一个FinoChatClient。 如果你想使用自定义模块,只有直接构造这个对像才能使用。
 * 通常，只要使用 {@link createClient} , 并传入“合适的”默认值就可以了。
 * @constructor
 * @extends {external:EventEmitter}
 * @extends {module:base-apis~FinoChatBaseApis}
 *
 * @param {Object}  opts FinoChatClient的配置属性.
 * @param {string}  opts.appKey Required. FinoChatSDK的appkey
 * @param {boolean} opts.debug Optinal. 开启调试模式,调用的所有api的返回值会在客户端alert出来，若要查看传入的参数，可以在pc端打开，参数信息会通过log打出，仅在pc端时才会打印
 */
function FinoChatClient(opts) {
  log.debug('DEBUG: FinoChatClient options', opts);
  FinoChatBaseApis.call(this, opts);
}

utils.inherits(FinoChatClient, EventEmitter);
utils.extend(FinoChatClient.prototype, FinoChatBaseApis.prototype);

/**
 * 初始化就绪回调
 * @param {module:client.callback} callback Optional.
 */
FinoChatClient.prototype.ready = function (callback) {
  jsBridgeEvent.on('jsBridgeReady', function () {
    log.debug('DEBUG: jsBridgeReady');
    callback();
  });
};
/**
 * 出错回调
 * @param {module:client.callback} callback Optional.
 */
FinoChatClient.prototype.error = function (callback) {};

/**
 * 检查客户支持API
 * @param {module:client.callback} callback Optional.
 */
FinoChatClient.prototype.checkJsApi = function (opt) {};

/** */
module.exports = FinoChatClient;
//# sourceMappingURL=client.js.map