"use strict";

/**
 * 这是一个内部模块. 查看 {@link FinoChatClient} 这个对外公共类.
 * @module jsbridge
 */
var utils = require("./utils");
var log = require("./log");
var jsBridgeEvent = require("./events").jsBridgeEvents;

/**
 * 创建一个JSBridge
 * 用于移动端浏览器<->Js之间的bridge通信
 * @constructor
 * */
function JSBridge() {
    if (utils.getBrowser().versions.ios) {
        log.debug("DEBUG JSBridge: iOS");
        utils.setupWebViewJavascriptBridge(function (bridge) {
            this.jsBridge = bridge;
            jsBridgeEvent.emit('jsBridgeReady', bridge);

            this.jsBridge.registerHandler('web_reloadPage', function (data, responseCallback) {
                window.reloadPageCallback && window.reloadPageCallback(data);
            });
        }.bind(this));
    } else if (utils.getBrowser().versions.android) {
        log.debug("DEBUG JSBridge: Android");
        utils.connectWebViewJavascriptBridge(function (bridge) {
            bridge.init(function (message, responseCallback) {
                var data = {};
                responseCallback(data);
            });
            this.jsBridge = bridge;
            jsBridgeEvent.emit('jsBridgeReady', bridge);

            this.jsBridge.registerHandler('cameraBack', function (data, responseCallback) {
                window.cameraCallback && window.cameraCallback(data);
            });

            this.jsBridge.registerHandler('getApproversBack', function (data, responseCallback) {
                window.getApproversCallback && window.getApproversCallback(data);
            });

            this.jsBridge.registerHandler('web_reloadPage', function (data, responseCallback) {
                window.reloadPageCallback && window.reloadPageCallback(data);
            });
        }.bind(this));
    } else {
        log.debug(utils.getBrowser().versions);
        log.debug(utils.getBrowser());
    }
}

/**
 * 获取jsBridge对像
 * @private
 * @param {module:client.callback} callback Optional.
 */
JSBridge.prototype.getJSBridge = function () {
    return this.jsBridge;
};

module.exports = JSBridge;
//# sourceMappingURL=jsbridge.js.map