'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

var _mop_bridge = require('./mop_bridge');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 这是MOPClient模板 目前仅用于
 * @module mop
 */

var currentRoute = '';

/**
 * MOPClient类,提供MOP小程序WebView的JSSDK交互功能
 * 代码示例:
 * <code>
 window.fc.miniProgram.navigateBack({
   success: function(res) {
     console.log('navigateBack callback:');
     console.log(res);
     alert(JSON.stringify(res))
   }
 });
 * </code>
 */

var eventHandlers = [];

var MOPClient = function () {
  function MOPClient() {
    (0, _classCallCheck3.default)(this, MOPClient);

    _log2.default.debug('MOPClient');
    try {
      (0, _mop_bridge.subscribe)('webviewEvent', function (data) {
        eventHandlers.forEach(function (handler) {
          handler(data.data);
        });
      });
      (0, _mop_bridge.subscribe)('webviewCurrentRoute', function (data) {
        currentRoute = data.route;
      });
      (0, _mop_bridge.publish)('onWebviewEvent', {
        key: 'getCurrentRoute'
      });
    } catch (e) {}
  }
  /**
   * 关闭当前页面，返回上一页面或多级页面。
   * @param {Object} params {
   *  delta: 1,
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */


  (0, _createClass3.default)(MOPClient, [{
    key: 'navigateBack',
    value: function navigateBack() {
      var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var data = params || {};
      data.name = 'navigateBack';
      data.arg = {
        delta: params.delta || 1
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 保留当前页面，跳转到应用内的某个页面。但是不能跳到 tabbar 页面。
     * @param {Object} params {
     *  url: '',
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'navigateTo',
    value: function navigateTo(params) {
      var data = params || {};
      if (!params.url) {
        throw new Error('Invalid parameter url');
      }
      params.url = _utils2.default.getRealRoute(currentRoute, params.url);
      params.url = _utils2.default.encodeUrlQuery(params.url);
      data.name = 'navigateTo';
      data.arg = {
        url: params.url || ''
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }

    /**
     * 关闭当前页面，跳转到应用内的某个页面。但是不允许跳转到 tabbar 页面。
     * @param {Object} params {
     *  url: '',
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'redirectTo',
    value: function redirectTo(params) {
      var data = params || {};
      if (!params.url) {
        throw new Error('Invalid parameter url');
      }
      params.url = _utils2.default.getRealRoute(currentRoute, params.url);
      params.url = _utils2.default.encodeUrlQuery(params.url);
      data.name = 'redirectTo';
      data.arg = {
        url: params.url || ''
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }

    /**
     * 跳转到 tabBar 页面，并关闭其他所有非 tabBar 页面。
     * @param {Object} params {
     *  url: '',
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'switchTab',
    value: function switchTab(params) {
      var data = params || {};
      if (!params.url) {
        throw new Error('Invalid parameter url');
      }
      params.url = _utils2.default.getRealRoute(currentRoute, params.url);
      params.url = _utils2.default.encodeUrlQuery(params.url);
      data.name = 'switchTab';
      data.arg = {
        url: params.url || ''
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 关闭所有页面，打开到应用内的某个页面
     * @param {Object} params {
     *  url: '',
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'reLaunch',
    value: function reLaunch(params) {
      var data = params || {};
      if (!params.url) {
        throw new Error('Invalid parameter url');
      }
      params.url = _utils2.default.getRealRoute(currentRoute, params.url);
      params.url = _utils2.default.encodeUrlQuery(params.url);
      data.name = 'reLaunch';
      data.arg = {
        url: params.url || ''
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 发送消息
     * @param {Object} params {
     *  data: '',
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'postMessage',
    value: function postMessage(params) {
      var data = params || {};
      data.name = 'postMessage';
      data.arg = params.data || {};
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 取当前运行环境
     * @param {Object} params {
     *  success: function(res) {}
     * }
     */

  }, {
    key: 'getEnv',
    value: function getEnv(params) {
      if (params && (typeof params === 'undefined' ? 'undefined' : (0, _typeof3.default)(params)) === 'object' && typeof params.success === 'function') {
        params.success({
          env: window.__fcjs_environment || {}
        });
      } else if (typeof params === 'function') {
        params({
          env: window.__fcjs_environment || {}
        });
      } else {
        console.log('params invalid');
      }
    }
    /**
     * 下载文件
     * @param {Object} params {
     *  url: '',
     *  header: '',
     *  timeout: 3000,
     *  filePath: ''
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'downloadFile',
    value: function downloadFile(params) {
      var data = params || {};
      data.name = 'downloadFile';
      data.arg = {
        url: params.url,
        header: params.header,
        timeout: params.timeout,
        filePath: params.filePath
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 打开文件，预览文件
     * @param {*} params {
     *  fileType: 'pdf',
     *  filePath: ''
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'openDocument',
    value: function openDocument(params) {
      var data = params || {};
      data.name = 'openDocument';
      data.arg = {
        filePath: params.filePath,
        fileType: params.fileType
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 选择图片
     * @param {*} params {
     * count: 1, // 默认9
     * sizeType: ['original', 'compressed'], // 可以指定是原图还是压缩图，默认二者都有
     * sourceType: ['album', 'camera'], // 可以指定来源是相册还是相机，默认二者都有
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'chooseImage',
    value: function chooseImage(params) {
      var data = params || {};
      data.name = 'chooseImage';
      data.arg = {
        count: params.count || 9,
        sizeType: params.sizeType || ['original', 'compressed'],
        sourceType: params.sourceType || ['album', 'camera']
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 获取本地图片 base64
     * @param {*} params
     * path: 本地图片路径
     */

  }, {
    key: 'getLocalImgData',
    value: function getLocalImgData(params) {
      var data = params || {};
      data.name = 'getLocalImgData';
      data.arg = {
        path: params.path
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 获取小程序信息
     * @param {callback} cb function(res) {}
     */

  }, {
    key: 'getAppletInfo',
    value: function getAppletInfo(params) {
      var data = params || {};
      data.name = 'getAppletInfo';
      data.arg = {};
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 关闭小程序
     */

  }, {
    key: 'close',
    value: function close(params) {
      var data = params || {};
      data.name = 'close';
      data.arg = {};
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 小程序跳转
     * @param {*} params {
     * appId: string, // 要打开的小程序 appId
     * path: string, //打开的页面路径
     * extraData: Object, // 需要传递给目标小程序的数据,App.onLaunch，App.onShow 中获取到这份数据
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'navigateToMiniProgram',
    value: function navigateToMiniProgram(params) {
      var data = params || {};
      data.name = 'navigateToMiniProgram';
      data.arg = {
        appId: params.appId,
        path: params.path,
        extraData: params.extraData
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 返回上一个小程序
     * @param {*} params {
     * extraData: Object, // 需要传递给目标小程序的数据,App.onLaunch，App.onShow 中获取到这份数据
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'navigateBackMiniProgram',
    value: function navigateBackMiniProgram(params) {
      var data = params || {};
      data.name = 'navigateBackMiniProgram';
      data.arg = {
        extraData: params.extraData
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 判断 H5 是否能返回上一页
     * @param {Object} params
     */

  }, {
    key: 'canGoBack',
    value: function canGoBack(params) {
      var data = params || {};
      data.name = 'canGoBack';
      data.arg = {};
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
    * 判断小程序页面是否能返回上一页
    * @param {Object} params
    */

  }, {
    key: 'canNavigateBack',
    value: function canNavigateBack(params) {
      var data = params || {};
      data.name = 'canNavigateBack';
      data.arg = {};
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
    * webview 页面设置 navigation title
    * @param {Object} params
    *   title: String
    * }
    */

  }, {
    key: 'setNavigationBarTitle',
    value: function setNavigationBarTitle(params) {
      var data = params || {};
      data.name = 'setNavigationBarTitle';
      data.arg = {
        title: String(params.title || '')
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * @description 获取本地缓存
     * @param {Object} params
     * @param {String} params.key
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     */

  }, {
    key: 'getStorage',
    value: function getStorage(params) {
      if (typeof params.key !== 'string') {
        throw new Error('Invalid parameter key');
      }
      var data = params || {};
      data.name = 'getStorage';
      data.arg = {
        key: params.key
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data, {
        beforeSuccess: function beforeSuccess(res) {
          if (typeof res.data === 'string') {
            try {
              var result = JSON.parse(res.data);
              if (result._type) {
                res.data = result.data;
              } else {
                res.data = result;
              }
              if (result._type === 'Date') {
                res.data = new Date(res.data);
              }
            } catch (error) {}
          }
          return res;
        }
      });
    }

    /**
     * @description 获取本地缓存的数据
     * @param {Object} params
     * @param {String} params.key
     * @param {String} params.data
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     */

  }, {
    key: 'setStorage',
    value: function setStorage(params) {
      if (typeof params.key !== 'string') {
        console.error('parameters error');
        return;
      }
      var _type = (0, _typeof3.default)(params.data);
      if (params.data instanceof Date) {
        _type = 'Date';
      }
      try {
        params.data = (0, _stringify2.default)({
          data: params.data,
          _type: _type
        });
      } catch (error) {
        console.error('parameters data error');
        return;
      }
      var data = params || {};
      data.name = 'setStorage';
      data.arg = {
        key: params.key,
        data: params.data
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }

    /**
     * @description 移除本地缓存
     * @param {Object} params
     * @param {String} params.key
     * success: function(res) {},
     * fail: function(res) {},
     * complete: function(res) {},
     */

  }, {
    key: 'removeStorage',
    value: function removeStorage(params) {
      if (typeof params.key !== 'string') {
        console.error('parameters error');
        return;
      }
      var data = params || {};
      data.name = 'removeStorage';
      data.arg = {
        key: params.key
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * @description 清除所有的缓存
     * success: function(res) {},
     * fail: function(res) {},
     * complete: function(res) {},
     */

  }, {
    key: 'clearStorage',
    value: function clearStorage(params) {
      var data = params || {};
      data.name = 'clearStorage';
      data.arg = {};
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }

    /**
     * @description 获取储存的信息
     * success: function(res) {},
     * fail: function(res) {},
     * complete: function(res) {},
     */

  }, {
    key: 'getStorageInfo',
    value: function getStorageInfo(params) {
      var data = params || {};
      data.name = 'getStorageInfo';
      data.arg = {};
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }
    /**
     * 调用app原生接口
     * @param {*} params {
     *  name: String, // 原生APP方法，命名规范，加【 user_define_ 】前缀
     *  data: Object, // 需要传递给原生APP的数据
     *  success: function(res) {},
     *  fail: function(res) {},
     *  complete: function(res) {},
     * }
     */

  }, {
    key: 'callHandler',
    value: function callHandler(params) {
      var data = params || {};
      (0, _mop_bridge.invokeMethod)(params.name, data);
    }
    /**
     * @description 获取当前地理位置信息
     * @param {Object} params
     * @param {String}  params.type 目前不管该参数如何，都会返回gcj02坐标
     * @param {Boolean}  params.altitude 传入 true 会返回高度信息，由于获取高度需要较高精确度，会减慢接口返回速度
     */

  }, {
    key: 'getLocation',
    value: function getLocation(params) {
      var data = params || {};
      data.name = 'getLocation';
      data.arg = {
        type: params.type,
        altitude: params.altitude
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }

    /**
     * @description 打开地图位置
     * @param {Object} params
     * @param {Number} params.latitude 目标地纬度
     * @param {Number} params.longitude 目标地经度
     */

  }, {
    key: 'chooseLocation',
    value: function chooseLocation(params) {
      var data = params || {};
      data.name = 'chooseLocation';
      data.arg = {
        latitude: params.latitude,
        longitude: params.longitude
      };
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }

    /**
    * @description 请求 api
    * @param {Object}} params
    */

  }, {
    key: 'request',
    value: function request() {
      var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      if (!params.url || typeof params.url !== 'string') {
        throw new Error('Invalid parameter url');
      }

      var headerType = _utils2.default.getDataType(params.header);
      params.header = params.header || {};
      params.header = _utils2.default.convertObjectValueToString(params.header);
      if (headerType !== 'Undefined' && headerType !== 'Object') {
        console.warn('request: header must be an object');
        params.header = {};
      }
      params.header = (0, _keys2.default)(params.header).reduce(function (res, cur) {
        cur.toLowerCase() === 'content-type' ? res[cur.toLowerCase()] = params.header[cur] : res[cur] = params.header[cur];
        return res;
      }, {});

      params.method && (params.method = params.method.toUpperCase());
      var headers = params.header || {};
      var requestMethod = typeof params.method === 'string' ? params.method.toUpperCase() : 'GET';
      var data = '';
      params.dataType = params.dataType || 'json';
      headers['content-type'] = headers['content-type'] || 'application/json';
      data = !params.data ? '' : typeof params.data !== 'string' ? headers['content-type'].indexOf('application/x-www-form-urlencoded') > -1 ? _utils2.default.urlEncodeFormData(params.data, !0) : headers['content-type'].indexOf('application/json') > -1 ? (0, _stringify2.default)(params.data) : (0, _typeof3.default)(params.data) === 'object' ? (0, _stringify2.default)(params.data) : data.toString() : params.data;

      if (requestMethod === 'GET') {
        params.url = _utils2.default.addQueryStringToUrl(params.url, params.data);
      }

      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', {
        name: 'request',
        arg: {
          url: params.url,
          data: data,
          header: headers,
          method: requestMethod,
          dataType: params.dataType || 'json',
          responseType: params.responseType,
          taskId: String(Math.random())
        },
        success: params.success,
        fail: params.fail,
        complete: params.complete
      }, {
        beforeSuccess: function beforeSuccess(res) {
          if (res.isBase64) {
            if (params.responseType && params.responseType.toLowerCase() === 'arraybuffer') {
              res.data = _utils2.default.base64ToArrayBuffer(res.data);
            } else {
              res.data = _utils2.default.atob(res.data);
            }
          }
          if (params.dataType === 'json') {
            try {
              res.data = JSON.parse(res.data);
            } catch (e) {}
          }
          res.statusCode = parseInt(res.statusCode);
        }
      });
    }
  }, {
    key: 'getSuperviseInfo',
    value: function getSuperviseInfo(params) {
      var data = params || {};
      data.name = 'getSuperviseInfo';
      data.arg = {};
      (0, _mop_bridge.invokeMethod)('invokeMiniProgramAPI', data);
    }

    /**
     * 注册小程序onShow时回调
     * @param {*} params {
     *
     * }
     */

  }, {
    key: 'onShow',
    value: function onShow(params) {
      var data = params || {};
      (0, _mop_bridge.subscribe)('pageonshow', data);
    }
  }, {
    key: 'onHide',
    value: function onHide(params) {
      var data = params || {};
      (0, _mop_bridge.subscribe)('pageonhide', data);
    }
  }, {
    key: 'onUnload',
    value: function onUnload(params) {
      var data = params || {};
      (0, _mop_bridge.webSubscribe)('pageonunload', data);
    }

    /**
     * H5 调用原生方法
     * @param {String} name
     * @param {Object} params
     * @param {Funciton} callback
     */

  }, {
    key: 'callNativeAPI',
    value: function callNativeAPI(name, params, callback) {
      (0, _mop_bridge.invoke)(name, params, callback);
    }

    /**
     * 注册 H5 方法供原生调用
     * @param {String} name 
     * @param {Function} handler 
     */

  }, {
    key: 'registNativeAPIHandler',
    value: function registNativeAPIHandler(name, handler) {
      (0, _mop_bridge.webSubscribe)(name, handler);
    }

    /**
     * 监听小程序发送的事件
     * @param {*} handler 
     */

  }, {
    key: 'onWebviewEvent',
    value: function onWebviewEvent(handler) {
      if (typeof handler === 'function') {
        eventHandlers.push(handler);
      }
    }

    /**
     * 移除监听小程序发送的事件
     * @param {*} handler 
     */

  }, {
    key: 'offWebviewEvent',
    value: function offWebviewEvent(handler) {
      if (typeof handler === 'function') {
        eventHandlers = eventHandlers.filter(function (f) {
          return f !== handler;
        });
      }
    }

    /**
     * 触发 webview 标签绑定的 bindevent 方法
     * @param {*} data 
     */

  }, {
    key: 'sendWebviewEvent',
    value: function sendWebviewEvent(data) {
      (0, _mop_bridge.publish)('onWebviewEvent', {
        key: 'sendEvent',
        params: data
      });
    }
  }]);
  return MOPClient;
}();

window.subscribeHandler = _mop_bridge.subscribeHandler;
window.fc = {
  miniProgram: new MOPClient()
};
window.ft = window.fc;
window.addEventListener('message', function (e) {
  try {
    var _JSON$parse = JSON.parse(e.data),
        callbackId = _JSON$parse.callbackId,
        params = _JSON$parse.params;

    window.FinChatJSBridge.webInvokeCallbackHandler(callbackId, params);
  } catch (e) {}
}, false);
module.exports = MOPClient;
//# sourceMappingURL=mop.js.map