'use strict';

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _module$exports;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var userAgent = window.navigator.userAgent;
// const isAndroid = userAgent.indexOf('Android') !== -1;
var isIOS = (userAgent.indexOf('iPhone') !== -1 || userAgent.indexOf('iPad') !== -1) && !window.isIDE;
var isWin = userAgent.indexOf('Windows') !== -1 && !window.isIDE;

// invoke callbaks
var callbacks = {};
var callbackIndex = 0;

// subscribe handlers
var handlers = {};
var eventPrefix = 'custom_event_';

var invokeHandler = function invokeHandler(event, paramsString, callbackId) {
  if (isIOS) {
    window.webkit.messageHandlers.webInvokeHandler.postMessage({
      C: event,
      paramsString: paramsString,
      callbackId: callbackId
    });
  } else if (isWin) {
    var payload = {
      C: event,
      paramsString: paramsString,
      callbackId: callbackId,
      type: 'invoke'
    };
    window.FinClipExecuteCommandAsync({
      request: (0, _stringify2.default)(payload),
      persistent: false,
      onSuccess: function onSuccess() {},
      onFailure: function onFailure() {}
    });
  } else {
    var jsCoreHandleResult = FinChatJSCore.webInvokeHandler(event, paramsString, callbackId);
    if (typeof jsCoreHandleResult !== 'undefined' && typeof callbacks[callbackId] === 'function' && jsCoreHandleResult !== '') {
      try {
        jsCoreHandleResult = JSON.parse(jsCoreHandleResult);
      } catch (e) {
        jsCoreHandleResult = {};
      }
      callbacks[callbackId](jsCoreHandleResult), delete callbacks[callbackId];
    }
  }
};

var invoke = function invoke(event, params, callback) {
  // postMessage
  var paramsString = (0, _stringify2.default)(params || {});
  var callbackId = ++callbackIndex;
  callbacks[callbackId] = callback;
  invokeHandler(event, paramsString, callbackId);
};
var webInvokeCallbackHandler = function webInvokeCallbackHandler(callbackId, params) {
  var callback = callbacks[callbackId];
  for (var i = 0; i < window.frames.length; i++) {
    window.frames[i].postMessage((0, _stringify2.default)({ callbackId: callbackId, params: params }), '*');
  }
  if (typeof callback === 'function') callback(params);
  delete callbacks[callbackId];
};

var webSubscribe = function webSubscribe(eventName, handler) {
  handlers[eventName] = handler;
};

var subscribe = function subscribe(eventName, handler) {
  handlers[eventPrefix + eventName] = handler;
};;

var subscribeHandler = function subscribeHandler(eventName, data, webviewId, reportParams) {
  // 执行注册的回调
  console.log('subscribeHandler', eventName);
  var handler = handlers[eventName];
  if (typeof handler === 'function') {
    return handler(data, webviewId, reportParams);
  }
};

var publish = function publish(eventName, params, webviewIds) {
  console.log(eventName, params, webviewIds);
  // publishHandler
  webviewIds = webviewIds || [];
  var paramsString = '';
  var event = eventName;
  paramsString = (0, _stringify2.default)(params);
  webviewIds = (0, _stringify2.default)(webviewIds);
  publishHandler(event, paramsString, webviewIds);
};

var publishHandler = function publishHandler(event, paramsString, webviewIds) {
  // publishHandler
  if (isIOS) {
    window.webkit.messageHandlers.publishHandler.postMessage({
      event: event,
      paramsString: paramsString,
      webviewIds: webviewIds
    });
  } else if (isWin) {
    var payload = {
      event: event,
      paramsString: paramsString,
      webviewIds: webviewIds,
      type: 'publish'
    };
    window.FinClipExecuteCommandAsync({
      request: (0, _stringify2.default)(payload),
      persistent: false,
      onSuccess: function onSuccess(res) {
        // _this.callbacks[callbackId](res)
        // delete _this.callbacks[callbackId]
      },
      onFailure: function onFailure(code, msg) {}
    });
    // window.chrome.webview.postMessage({
    //   event: event,
    //   paramsString: paramsString,
    //   webviewIds: webviewIds,
    //   type: 'publish', 
    // });
  } else {
    FinChatJSCore.publishHandler(event, paramsString, webviewIds);
  }
};

var invokeMethod = function invokeMethod(apiName, opts, innerCb) {
  var options = opts || {};
  var innerFns = innerCb || {};
  var params = {};

  for (var i in options) {
    if ('function' == typeof options[i]) {
      params[i] = options[i];
      delete options[i];
    }
  }
  var sysEventFns = {};
  for (var s in innerFns) {
    if ('function' == typeof innerFns[s]) {
      sysEventFns[s] = innerFns[s];
    }
  }

  invoke(apiName, options, function (res) {
    res.errMsg = res.errMsg || apiName + ':ok';
    var isOk = 0 === res.errMsg.indexOf(apiName + ':ok');
    var isCancel = 0 === res.errMsg.indexOf(apiName + ':cancel');
    var isFail = 0 === res.errMsg.indexOf(apiName + ':fail');
    if (isOk) {
      'function' == typeof sysEventFns.beforeAll && sysEventFns.beforeAll(res);
      'function' == typeof sysEventFns.beforeSuccess && sysEventFns.beforeSuccess(res);
      'function' == typeof params.success && params.success(res);
      'function' == typeof sysEventFns.afterSuccess && sysEventFns.afterSuccess(res);
    } else if (isCancel) {
      res.errMsg = res.errMsg.replace(apiName + ':cancel', apiName + ':fail cancel'), 'function' == typeof params.fail && params.fail(res), 'function' == typeof sysEventFns.beforeCancel && sysEventFns.beforeCancel(res), 'function' == typeof params.cancel && params.cancel(res), 'function' == typeof sysEventFns.afterCancel && sysEventFns.afterCancel(res);
    } else if (isFail) {
      'function' == typeof sysEventFns.beforeFail && sysEventFns.beforeFail(res), 'function' == typeof params.fail && params.fail(res);
      'function' == typeof sysEventFns.afterFail && sysEventFns.afterFail(res);
    }
    'function' == typeof params.complete && params.complete(res), 'function' == typeof sysEventFns.afterAll && sysEventFns.afterAll(res);
  });
};

window.FinChatJSBridge = {
  invoke: invoke,
  invokeCallbackHandler: webInvokeCallbackHandler,
  webInvokeCallbackHandler: webInvokeCallbackHandler,
  subscribe: subscribe,
  subscribeHandler: subscribeHandler,
  // Mark: 兼容的调用方式
  webSubscribeCallBackHandler: subscribeHandler,
  publish: publish,
  publishHandler: publishHandler,
  webSubscribe: webSubscribe
};
module.exports = (_module$exports = {
  invoke: invoke,
  invokeCallbackHandler: webInvokeCallbackHandler,
  webInvokeCallbackHandler: webInvokeCallbackHandler,
  subscribe: subscribe,
  subscribeHandler: subscribeHandler,
  // Mark: 兼容的调用方式
  webSubscribeCallBackHandler: subscribeHandler,
  publish: publish,
  publishHandler: publishHandler
}, (0, _defineProperty3.default)(_module$exports, 'subscribeHandler', subscribeHandler), (0, _defineProperty3.default)(_module$exports, 'invokeMethod', invokeMethod), (0, _defineProperty3.default)(_module$exports, 'webSubscribe', webSubscribe), _module$exports);
//# sourceMappingURL=mop_bridge.js.map